@@ -1,4 +1,4 @@
-from mltraq.options import options
+from mltraq.default_options import options
 from mltraq.run import Run, Runs
 from mltraq.session import Session as create_session
 from mltraq.session import experiment
