@@ -1,21 +1,20 @@
 import copy
+import logging
 from contextlib import contextmanager
 from typing import Callable, List, Union
 
-import joblib
 import pandas as pd
 from sqlalchemy.orm import load_only
 
-from mltraq.options import options
+from mltraq import options
 from mltraq.run import Run, Runs, get_params_list
-from mltraq.storage import models, serialization
-from mltraq.storage.database import Database, next_ulid, pandas_query, sanitize_table_name
-from mltraq.utils import log
-from mltraq.utils.dicts import ObjectDictionary
+from mltraq.storage import models, pyson, serialization
+from mltraq.storage.database import Database, next_ulid, pandas_query, sanitize_table_name, short_uuid
+from mltraq.utils.bunch import Bunch
 from mltraq.utils.enums import IfExists, enforce_enum
 from mltraq.utils.frames import json_normalize, reorder_columns
-from mltraq.utils.log import logger
-from mltraq.utils.uuid import uuid3words
+
+log = logging.getLogger(__name__)
 
 
 class PickleNotFoundException(Exception):
@@ -62,13 +61,15 @@ class Experiment:
     # model_cls is the SQLAlchemy model mapped to this class.
     model_cls = models.Experiment
 
+    __slots__ = ("id_experiment", "db", "name", "fields", "runs")
+    __state__ = ("id_experiment", "name", "fields", "runs")
+
     def __init__(
         self,
         db: Database = None,
         id_experiment: str = None,
         name: str = None,
         fields: dict = None,
-        properties: dict = None,
         runs: List[Run] = None,
     ):
         """Create a new experiment.
@@ -78,7 +79,6 @@ class Experiment:
             id_experiment (str, optional): ID of the experiment to be used. Defaults to None.
             name (str, optional): Name of the experiment. Defaults to None.
             fields (dict, optional): Fixed experiment fields to be tracked. Defaults to None.
-            properties (dict, optional): Experiment properties, used to hold meta-attributes. Defaults to None.
             runs (List[Run], optional): List of runs belonging to the experiment. Defaults to None.
         """
 
@@ -86,35 +86,19 @@ class Experiment:
         self.id_experiment = next_ulid() if id_experiment is None else id_experiment
 
         if name is None:
-            name = uuid3words(self.id_experiment)
+            name = short_uuid(self.id_experiment)
         self.name = name
-        self.fields = ObjectDictionary(fields)
+        self.fields = Bunch(fields)
         self.runs = Runs(runs)
 
-        # if param_grid is not None:
-        #    self.runs.add_grid(param_grid, kwargs=kwargs, steps=steps)
+    def __getstate__(self):
+        state = {key: getattr(self, key) for key in self.__state__}
+        return state
 
-        # Populate properties
-        self.properties = ObjectDictionary(properties)
-        # if "env" not in self.properties:
-        # If the experiment is loaded, then "env" is already populated and we retain it.
-        # self.properties["env"] = get_environment()
-
-    def __reduce__(self):
-        """
-        Make the expriment pickable, avoiding unpickable objects in SQLAlchemy.
-
-        Returns:
-            Experiment: An unpickled Experiment object.
-        """
-        return self.__class__, (
-            None,
-            self.id_experiment,
-            self.name,
-            self.fields,
-            self.properties,
-            self.runs,
-        )
+    def __setstate__(self, state):
+        self.db = None
+        for k, v in state.items():
+            self.__setattr__(k, v)
 
     @contextmanager
     def run(self):
@@ -151,7 +135,13 @@ class Experiment:
             param_grid = list(param_grid)
 
         for params in param_grid:
-            self.runs.add(Run(params=params))
+            self.add_run(**params)
+
+        if len(param_grid) == 0:
+            raise RuntimeError(
+                "Invalid parameters grid, no added runs. This is likely due to a parameter associated to zero possible"
+                " values."
+            )
 
         return self
 
@@ -173,7 +163,6 @@ class Experiment:
 
         return sanitize_table_name(f"{options.get('db.experiment_tableprefix')}{self.name}")
 
-    @log.default_exception_handler
     def query(self, query: str = "SELECT * FROM {table_name}") -> pd.DataFrame:
         """Query the experiment's table
 
@@ -197,27 +186,27 @@ class Experiment:
 
         return df
 
-    def load_runs(self, run_columns):
-        """Load runs for experiment from db, considering run_columns.
+    def load_runs(self, runs_meta):
+        """Load runs for experiment from db, considering runs_meta.
 
         Args:
-            run_columns (_type_): dictionary that specifies which columns have
+            runs_meta (_type_): dictionary that specifies which columns have
                 been serialized or not.
 
         Returns:
             _type_: _description_
         """
-        logger.info("Loading runs")
+        log.debug("Loading runs")
         # Retrieve all columns
         df = self.query(f"select * from {self.tablename()}")  # noqa
 
         # Take care of deserialization
-        for col_name in run_columns["serialized"]:
-            df[col_name] = df[col_name].map(serialization.deserialize)
+        for col_name in runs_meta["serialized"]:
+            df[col_name] = df[col_name].map(pyson.deserialize)
 
         # Reconstruct fields
         def series_to_run(row: pd.Series):
-            fields = row[run_columns["serialized"] + run_columns["non_serialized"]].to_dict()
+            fields = row[runs_meta["serialized"] + runs_meta["non-serialized"]].to_dict()
             run = Run(id_run=row["id_run"], fields=fields)
             return run
 
@@ -237,7 +226,7 @@ class Experiment:
         experiment.id_experiment = next_ulid()
 
         if name is None:
-            name = uuid3words(self.id_experiment)
+            name = short_uuid(self.id_experiment)
 
         experiment.name = name
 
@@ -258,7 +247,7 @@ class Experiment:
         Returns:
             _type_: _description_
         """
-        logger.info(f"Loading experiment '{name}' (pickle)")
+        log.debug(f"Loading experiment '{name}' (pickle)")
 
         with db.session() as session:
             record = (
@@ -304,16 +293,15 @@ class Experiment:
         if pickle:
             return cls.load_pickle(db, name)
 
-        logger.info(f"Loading experiment '{name}'")
+        log.debug(f"Loading experiment '{name}'")
 
         # Create a new session
         columns = [
             Experiment.model_cls.id_experiment,
             Experiment.model_cls.name,
-            Experiment.model_cls.run_count,
-            Experiment.model_cls.run_columns,
+            Experiment.model_cls.runs_size,
+            Experiment.model_cls.runs_meta,
             Experiment.model_cls.fields,
-            Experiment.model_cls.properties,
         ]
 
         with db.session() as session:
@@ -328,29 +316,27 @@ class Experiment:
                 db=db,
                 id_experiment=record.id_experiment,
                 name=record.name,
-                fields=serialization.deserialize(record.fields),
-                properties=serialization.deserialize(record.properties),
+                fields=pyson.deserialize(record.fields),
             )
 
-            if record.run_count > 0:
-                experiment.load_runs(run_columns=serialization.deserialize(record.run_columns))
+            if record.runs_size > 0:
+                experiment.load_runs(runs_meta=pyson.deserialize(record.runs_meta))
 
             return experiment
 
-    @log.default_exception_handler
     def execute(
         self,
         steps: Union[Callable, List[Callable]] = None,
-        kwargs=None,
-        backend=joblib.parallel.DEFAULT_BACKEND,
-        n_jobs=-1,
+        config=None,
+        backend=None,
+        n_jobs=None,
     ):
         """Execute the experiment, then retuning it.
 
         Args:
             steps (Union[Callable, List[Callable]], optional): Function(s) that will override the defined ones.
                 If the value is "all", the re-evaluation of all defined functions is forced. Defaults to None.
-            kwargs: Fixed arguments to pass to the run.
+            config: Fixed arguments to pass to the run.
             backend (_type_, optional): Joblib baackend. Defaults to joblib.parallel.DEFAULT_BACKEND.
             n_jobs (int, optional): Parallelization degree, defined as in Joblib. Defaults to -1.
 
@@ -358,13 +344,13 @@ class Experiment:
             Run: the same experiment, ready for chained operations.
         """
 
-        self.runs.execute(steps=steps, kwargs=kwargs, backend=backend, n_jobs=n_jobs)
+        self.runs.execute(steps=steps, config=config, backend=backend, n_jobs=n_jobs)
 
         return self
 
     def record(
         self,
-        run_columns=None,
+        runs_meta=None,
         store_pickle=None,
         enable_compression=None,
     ) -> models.Experiment:
@@ -383,10 +369,9 @@ class Experiment:
             id_experiment=self.id_experiment,
             name=self.name,
             fields=serialization.serialize(self.fields, enable_compression=enable_compression),
-            properties=serialization.serialize(self.properties, enable_compression=enable_compression),
             pickle=serialization.pickle_dumps(self) if store_pickle else None,
-            run_count=len(self.runs),
-            run_columns=serialization.serialize(run_columns, enable_compression=enable_compression),
+            runs_size=len(self.runs),
+            runs_meta=serialization.serialize(runs_meta, enable_compression=enable_compression),
         )
 
     def info(self):
@@ -398,31 +383,27 @@ class Experiment:
 
         if self.runs:
             run = self.runs.first()
-            run_kwargs = list(run.kwargs.keys())
+            run_config = list(run.config.keys())
             run_params = list(run.params.keys())
             run_fields = list(run.fields.keys())
-            run_steps = [func.__name__ for func in run.steps]
 
         else:
-            run_kwargs = []
+            run_config = []
             run_params = []
             run_fields = []
-            run_steps = []
 
         return pd.Series({
             "name": self.name,
             "id": self.id_experiment,
             "fields": list(self.fields.keys()),
-            "run_steps": run_steps,
             "run_fields": run_fields,
-            "run_kwargs": run_kwargs,
+            "run_config": run_config,
             "run_params": run_params,
-            "run_count": len(self.runs),
+            "runs_size": len(self.runs),
             "pickle_size": self.size("mb"),
             "table_name": self.tablename(),
         })
 
-    @log.default_exception_handler
     def persist(
         self,
         if_exists: IfExists = IfExists["fail"],
@@ -447,7 +428,7 @@ class Experiment:
         if enable_compression is None:
             enable_compression = options.get("serialization.enable_compression")
 
-        logger.info(f"Persisting experiment (table name: {self.tablename()})")
+        log.debug(f"Persisting experiment (table name: {self.tablename()})")
 
         if_exists = enforce_enum(if_exists, IfExists)
 
@@ -459,26 +440,26 @@ class Experiment:
             # Work on a copy, to avoid modifications a slided copy of the dataframe.
             df_runs = self.runs.df(max_level=0).copy()
             df_runs["id_experiment"] = self.id_experiment
-            df_runs, run_columns = serialization.serialize_df(
+            df_runs, runs_meta = serialization.serialize_df(
                 df_runs, ignore_columns=["id_run", "id_experiment"], enable_compression=enable_compression
             )
             df_runs = reorder_columns(df_runs, ["id_experiment", "id_run"])
         else:
             df_runs = pd.DataFrame(columns=["id_experiment", "id_run"])
-            run_columns = {"serialized": [], "non_serialized": [], "compression": enable_compression}
+            runs_meta = {"serialized": [], "non-serialized": [], "compression": enable_compression}
 
         # Insert row in "experiments" table.
 
         with self.db.session() as session:
             session.add(
-                self.record(run_columns=run_columns, store_pickle=store_pickle, enable_compression=enable_compression)
+                self.record(runs_meta=runs_meta, store_pickle=store_pickle, enable_compression=enable_compression)
             )
             session.commit()
 
         dtype = {
             **{"id_run": models.uuid_type},
             **{"id_experiment": models.uuid_type},
-            **{col_name: models.LargeBinary for col_name in run_columns["serialized"]},
+            **{col_name: models.LargeBinary for col_name in runs_meta["serialized"]},
         }
 
         # Create the experiment table.
@@ -486,7 +467,6 @@ class Experiment:
 
         return self
 
-    @log.default_exception_handler
     def delete(self, if_exists: IfExists = IfExists["fail"]) -> None:
         """Delete the expeirment.
 
@@ -564,7 +544,7 @@ class Experiment:
         return serialization.pickle_size(self, unit=unit)
 
     @classmethod
-    def ls(cls, db: Database, include_properties=False) -> pd.DataFrame:
+    def ls(cls, db: Database) -> pd.DataFrame:
         """List experiments in a database.
 
         Args:
@@ -575,10 +555,7 @@ class Experiment:
         """
 
         # Construct list of columns to retrieve
-        if include_properties:
-            columns = [Experiment.model_cls.id_experiment, Experiment.model_cls.name, Experiment.model_cls.properties]
-        else:
-            columns = [Experiment.model_cls.id_experiment, Experiment.model_cls.name]
+        columns = [Experiment.model_cls.id_experiment, Experiment.model_cls.name]
 
         with db.session() as session:
             df_experiments = pandas_query(
@@ -590,8 +567,4 @@ class Experiment:
                 lambda name: sanitize_table_name(f"{options.get('db.experiment_tableprefix')}{name}")
             )
 
-            if include_properties:
-                df_experiments["properties"] = df_experiments["properties"].map(serialization.deserialize)
-                df_experiments = serialization.explode_json_column(df_experiments, col_name="properties")
-
             return df_experiments
