@@ -1,15 +1,17 @@
 import copy
+import logging
 
 import pandas as pd
 
+from mltraq.default_options import doc_url, options
 from mltraq.experiment import Experiment
-from mltraq.options import options
 from mltraq.storage.database import Database
 from mltraq.utils.enums import IfExists, enforce_enum
-from mltraq.utils.log import default_exception_handler, init_logging, logger
 from mltraq.utils.text import stringify
 from mltraq.version import __version__
 
+log = logging.getLogger(__name__)
+
 
 class Session:
     """Instantiate the MLtraq handler, it can then be sued
@@ -17,6 +19,10 @@ class Session:
     they do work together nicely without interfering.
     """
 
+    # We want to have an immutable, explicit list of the class
+    # attributes. This is achieve with __slots__.
+    __slots__ = ("db",)
+
     def __init__(self, url: str = None, ask_password=False):
         """Create a new MLtraq handler.
 
@@ -28,9 +34,8 @@ class Session:
         if url is None:
             url = options.get("db.url")
 
-        init_logging()
         self.db = Database(url, ask_password=ask_password)
-        logger.info(f"MLtraq v{__version__} ({options.get('doc.url')}) initialized")
+        log.debug(f"MLtraq v{__version__} ({doc_url}) initialized")
 
     def _repr_html_(self):
         experiment_names = self.ls()["name"].tolist()
@@ -40,14 +45,13 @@ class Session:
             f" experiments({len(experiment_names)})={stringify(experiment_names)})"
         )
 
-    @default_exception_handler
     def add_experiment(self, name: str = None, **fields) -> Experiment:
         """Define a new experiment.
 
         Args:
             name (str, optional): Name of the experiment (must be unique). Defaults to None.
             steps (Union[Callable, List[Callable]], optional): Function(s) we want to apply to
-            the input parameters. Defaults to None. kwargs (dict, optional): Fixed arguments to
+            the input parameters. Defaults to None. config (dict, optional): Fixed arguments to
             the functions. Defaults to None. attributes (dict, optional): Fixed experiment properties
             to be tracked. Defaults to None. parameter_grid (List, optional): Variable parameters
             to pass to the functions. Defaults to None.
@@ -62,16 +66,14 @@ class Session:
             fields=fields,
         )
 
-    @default_exception_handler
-    def ls(self, include_properties=False) -> pd.DataFrame:
+    def ls(self) -> pd.DataFrame:
         """List the tracked experiments.
 
         Returns:
             pd.DataFrame: Pandas dataframe.
         """
-        return Experiment.ls(self.db, include_properties=include_properties)
+        return Experiment.ls(self.db)
 
-    @default_exception_handler
     def load(self, name: str = None, pickle=False) -> Experiment:
         """Load an experiment from the database and return it.
 
@@ -83,7 +85,6 @@ class Session:
         """
         return Experiment.load(self.db, name, pickle=pickle)
 
-    @default_exception_handler
     def persist(self, experiment: Experiment, if_exists: IfExists = "fail") -> Experiment:
         """Persist the experiment to the database (not necessarily
         the one used to track it initially).
@@ -122,7 +123,7 @@ class Session:
 
     def version(self):
         """Log MLtraq version"""
-        logger.info(f"MLtraq v{__version__}")
+        log.debug(f"MLtraq v{__version__}")
 
 
 def experiment(url: str = None, ask_password=False) -> Experiment:
