@@ -1,4 +1,5 @@
 import getpass
+import logging
 import re
 import uuid
 from functools import partial
@@ -10,13 +11,13 @@ from sqlalchemy.engine import make_url
 from sqlalchemy.exc import NoSuchTableError
 from sqlalchemy.orm import Query, sessionmaker
 from sqlalchemy.sql import text
-from ulid import monotonic as ulid
+from tqdm.auto import tqdm
 
-from mltraq.options import options
+from mltraq import options
 from mltraq.storage.models import Base
 from mltraq.utils.enums import IfExists
-from mltraq.utils.log import logger
-from mltraq.utils.progress import progress
+
+log = logging.getLogger(__name__)
 
 
 class Database:
@@ -75,7 +76,7 @@ class Database:
 
     def info(self):
         """Log some stats about the database."""
-        logger.info(f"Database: {self.url.render_as_string(hide_password=True)}")
+        log.debug(f"Database: {self.url.render_as_string(hide_password=True)}")
 
     def pandas_to_sql(self, df: pd.DataFrame, name: str, if_exists: IfExists, dtype: dict = None):
         """Insert a Pandas dataframe as a new database table.
@@ -141,7 +142,7 @@ class Database:
 
             session.execute(text("VACUUM"))
 
-        logger.info("VACUUM executed.")
+        log.debug("VACUUM executed.")
 
     def pandas(
         self,
@@ -223,7 +224,7 @@ def pandas_query(
     elif isinstance(query, str):
         query = sql.expression.text(query)
 
-    logger.debug(f"SQL: {query.compile(session.bind)}")
+    log.debug(f"SQL: {query.compile(session.bind)}")
 
     if use_tqdm:
         if tqdm_total is None:
@@ -234,6 +235,7 @@ def pandas_query(
         df_chunks = pd.read_sql_query(
             sql=query, con=session.connection(), chunksize=options.get("db.query_read_chunk_size")
         )
+
         funcs = [partial(lambda df_chunk: (len(df_chunk), df_chunk), df_chunk) for df_chunk in df_chunks]
         dfs = tqdm_chunk(funcs, tqdm_total)
 
@@ -241,16 +243,24 @@ def pandas_query(
     else:
         df = pd.read_sql_query(query, session.bind)
 
-    return df.apply(pd.to_numeric, errors="ignore")
+    return df
+
+
+# Dummy fixed incremental UUIDs, to be used only for generating predictable UUIDs for
+# examples and documentation
+next_fake_uuid_int = 176480063973334418222224317881181085494
 
 
 def next_ulid() -> str:
-    """Return the next monotonic ULID (https://github.com/ulid/spec) as a string.
+    """Return the next UUID as a string."""
 
-    Returns:
-        str: New ULID in UUID format, ready to be used as Experiment or Run ID.
-    """
-    return str(uuid.UUID(bytes=ulid.new().bytes))  # str(uuid.UUID(bytes=ulid.new().bytes))
+    global next_fake_uuid_int
+
+    if options.get("reproducibility.fake_incremental_uuids"):
+        next_fake_uuid_int += 1
+        return str(uuid.UUID(int=next_fake_uuid_int))
+    else:
+        return str(uuid.uuid4())
 
 
 def chunker(seq: List, size) -> List[List]:
@@ -282,7 +292,8 @@ def tqdm_chunk(
         List: List of return values from callables.
     """
 
-    pbar = progress(total=total)
+    pbar = tqdm(**(options.get("tqdm") | {"total": total}))
+
     pbar.clear()
 
     rets = []
@@ -295,3 +306,23 @@ def tqdm_chunk(
     pbar.close()
 
     return rets
+
+
+def short_uuid(n):
+    if isinstance(n, uuid.UUID):
+        n = n.int
+    elif isinstance(n, str):
+        n = uuid.UUID(n).int
+
+    expected_length = 6
+    padding = "0" * expected_length
+    alphabet = "123456789ACEFHJKLMNPRTUVWXY"
+    length = len(alphabet)
+    result = ""
+    remain = n
+    while remain > 0:
+        pos = remain % length
+        remain //= length
+        result += alphabet[pos]
+    result += padding
+    return result[:expected_length].lower()
