@@ -28,7 +28,7 @@ def stringify(d: object, max_len=200):
     return s
 
 
-def wprint(text):
+def wprint(text):  # XXX pprint ?
     """print string, wrapping it.
 
     Args:
